@@ -6,11 +6,10 @@ import timeit
 
 import pytest
 
+from vcd.common import Timescale, TimescaleMagnitude, TimescaleUnit
 from vcd.writer import (
     CompoundVectorVariable,
     ScopeType,
-    TimescaleMagnitude,
-    TimescaleUnit,
     Variable,
     VCDPhaseError,
     VCDWriter,
@@ -36,6 +35,7 @@ def test_vcd_init(capsys):
         ((1, 'ns'), '1 ns'),
         ('100ps', '100 ps'),
         ((TimescaleMagnitude.ten, TimescaleUnit.femtosecond), '10 fs'),
+        (Timescale(TimescaleMagnitude.hundred, TimescaleUnit.millisecond), '100 ms'),
     ],
 )
 def test_vcd_timescales(capsys, timescale, expected):
@@ -104,6 +104,15 @@ def test_vcd_change_after_close(capsys):
         vcd.flush()
 
 
+def test_vcd_alias_after_close(capsys):
+    vcd = VCDWriter(sys.stdout)
+    var = vcd.register_var('a', 'b', 'integer')
+    assert not split_lines(capsys)
+    vcd.close()
+    with pytest.raises(VCDPhaseError):
+        vcd.register_alias('c', 'd', var)
+
+
 def test_vcd_no_scopes(capsys):
     with VCDWriter(sys.stdout, date='today', version='some\nversion', comment='hello'):
         pass
@@ -169,6 +178,36 @@ def test_vcd_no_duplicates(capsys):
     ]
 
 
+def test_vcd_aliases(capsys):
+    with VCDWriter(sys.stdout, date='today') as vcd:
+        var = vcd.register_var('sss', 'nnn', 'integer', 32)
+        vcd.register_alias('sss', 'mmm', var)
+        vcd.register_alias('ttt', 'nnn', var)
+        vcd.change(var, 0, 'x')
+        vcd.change(var, 1, 10)
+        vcd.change(var, 2, 11)
+    assert split_lines(capsys) == [
+        '$date today $end',
+        '$timescale 1 us $end',
+        '$scope module sss $end',
+        '$var integer 32 ! nnn $end',
+        '$var integer 32 ! mmm $end',
+        '$upscope $end',
+        '$scope module ttt $end',
+        '$var integer 32 ! nnn $end',
+        '$upscope $end',
+        '$enddefinitions $end',
+        '#0',
+        '$dumpvars',
+        'bx !',
+        '$end',
+        '#1',
+        'b1010 !',
+        '#2',
+        'b1011 !',
+    ]
+
+
 def test_vcd_scopes(capsys):
     with VCDWriter(sys.stdout, date='today') as vcd:
         vcd.set_scope_type('eee.fff.ggg', ScopeType.task)
@@ -278,6 +317,20 @@ def test_vcd_late_registration(capsys):
             vcd.register_var('aaa.bbb', 'nn2', 'integer', 8)
 
 
+def test_vcd_late_alias_registration(capsys):
+    with VCDWriter(sys.stdout, date='today') as vcd:
+        var0 = vcd.register_var('aaa.bbb', 'nn0', 'integer', 8)
+        vcd.change(var0, 0, 123)
+
+        # Still at t0, registration okay...
+        vcd.register_alias('aaa.bbb', 'nn1', var0)
+
+        vcd.change(var0, 1, 210)
+
+        with pytest.raises(VCDPhaseError):
+            vcd.register_alias('aaa.bbb', 'nn2', var0)
+
+
 def test_vcd_missing_size(capsys):
     with VCDWriter(sys.stdout, date='today') as vcd:
         with pytest.raises(ValueError):
@@ -303,6 +356,16 @@ def test_vcd_duplicate_var_name(capsys):
             vcd.register_var('aaa.bbb', 'nn0', 'wire', 1)
 
 
+def test_vcd_duplicate_alias(capsys):
+    with VCDWriter(sys.stdout, date='today') as vcd:
+        var = vcd.register_var('aaa.bbb', 'nn0', 'integer', 8)
+        vcd.register_alias('aaa.bbb', 'nn1', var)
+        with pytest.raises(KeyError):
+            vcd.register_alias('aaa.bbb', 'nn0', var)
+        with pytest.raises(KeyError):
+            vcd.register_alias('aaa.bbb', 'nn1', var)
+
+
 def test_vcd_change_out_of_order(capsys):
     with VCDWriter(sys.stdout, date='') as vcd:
         var = vcd.register_var('scope', 'a', 'wire', 1)
@@ -860,11 +923,15 @@ def test_vcd_string_var(capsys):
         vcd.change(v0, 1, 'hello')
         vcd.change(v0, 2, '')
         vcd.change(v0, 3, 'world')
-        with pytest.raises(ValueError):
-            vcd.change(v0, 4, 'no string allowed')
-        vcd.change(v0, 4, None)
-        vcd.change(v0, 5, '!')
-        vcd.dump_off(6)
+        vcd.change(v0, 4, 'spaces are\tok')
+        vcd.change(v0, 5, 'newlines\r\ntoo')
+        vcd.change(v0, 6, 'slash\\slash')
+        vcd.change(v0, 7, None)
+        vcd.change(v0, 8, '!')
+        with pytest.raises(ValueError) as e:
+            vcd.change(v0, 9, 42)
+        assert 'Invalid string value' in e.value.args[0]
+        vcd.dump_off(8)
     expected = [
         '#0',
         '$dumpvars',
@@ -878,10 +945,15 @@ def test_vcd_string_var(capsys):
         '#3',
         'sworld !',
         '#4',
-        's !',
+        'sspaces\\x20are\\tok !',
         '#5',
-        's! !',
+        'snewlines\\r\\ntoo !',
         '#6',
+        'sslash\\\\slash !',
+        '#7',
+        's !',
+        '#8',
+        's! !',
         '$dumpoff',
         '$end',
     ]
@@ -892,7 +964,7 @@ def test_vcd_string_var(capsys):
 def test_execution_speed():
     """Manual test for how fast we can write to a VCD file
 
-    See https://github.com/SanDisk-Open-Source/pyvcd/issues/9
+    See https://github.com/westerndigitalcorporation/pyvcd/issues/9
 
     pytest -vvs -k test_execution_speed
 
